<?php
// $Id: Exporter.php,v 1.1.2.1 2011/01/04 18:25:40 pounard Exp $

/**
 * Helper class for exporting code.
 */
class Xoxo_Code_Exporter
{
  /**
   * Kind of registry pattern. There is no real use of this, exception for
   * chaining instanciation and object usage in code.
   * 
   * @return Xoxo_Code_Exporter
   *   Fresh new instance.
   */
  public static function createInstance() {
    return new Xoxo_Code_Exporter();
  }

  /**
   * Get default disclaimer.
   * 
   * @return string
   */
  protected function _getDefaultDisclaimer() {
    return <<<EOT
This is XoXo module generated import code. If you are planning to
put this code into a custom module hook implementation, please let
this message inside so the future developpers will known that this
has been generated.

READ THIS BEFORE ANY IMPORT ATTEMPT.

Remember that this code is being generated automatically and thus
if not fail-safe nor secure. Any object import should always be
tested on a secure development box, before any production site
import.

The importing facility may exist in the Drupal interface, you
have to use it wisely. Any malicious or miswrote code could be
imported, and cause the Drupal instance either to import wrong
or corrupted data, which could lead to either security breaches
or site being down with an ugly WOSD.

USE THIS AT YOUR OWN RISKS!
EOT;
  }

  /**
   * Export the given object to raw PHP code.
   * 
   * This is the method that specific factories should override. This simple
   * implementation will be enougth for objects using only options as storage.
   * 
   * @param Oox_Registrable_Interface|array $objects
   *   Any Oox_Registrable_Interface instance or an array of Oox_Registrable_Interface instances. If this
   *   array is keyed, keys will be used as variables names. Else variables'
   *   names will be generated.
   * @param string|array $disclaimer = TRUE
   *   (optional) If set, will use a different disclaimer than the original one.
   *   You can legally not display any disclaimer by setting an empty array or
   *   an empty string.
   *   See Xoxo_Code_Exporter::_appendComment() method signature for data you can
   *   give.
   * 
   * @return array
   *   An array of PHP code lines.
   * 
   * @throws Xoxo_Factory_Exception
   *   In case any error happens.
   */
  public function export($objects, $disclaimer = NULL) {
    // First of all, we need a new context, and a statement.
    $context = new Xoxo_Code_Scope();
    $header = new Xoxo_Code_Statement_Arbitrary;
    $statement = new Xoxo_Code_Statement_Arbitrary;

    // We need a statement to display.
    if (!is_array($objects)) {
      $objects = array($objects);
    }

    // Prepare statement disclaimer.
    if ($disclaimer === NULL) {
      $disclaimer = $this->_getDefaultDisclaimer();
    }

    // Prepare header statement.
    $header
      ->setContext($context)
      ->appendComment($disclaimer)
      ->appendEmptyLine();

    // Populate our new statement.
    $itemArray = $context->escapeVariable("items");

    $statement
      ->setContext($context)
      // Forces the header disclaimer to be on top.
      ->addPrefix($header)
      ->appendEmptyLine()
      ->appendComment(array(
        "This is your real object export.",
        "",
        "All the code above is about creating those objects dependencies.",
        "Do not omit any lines of this code, except if you know what you",
        "are really doing."
      ))
      ->appendEmptyLine()
      ->append($itemArray . " = array();");

    // Append our objects to the arbitrary statement.
    foreach ($objects as $object) {
      $statement->append($itemArray . "[] = " . $statement->escape($object) . ";");
    }

    return $statement
      ->append("return " . $itemArray . ";")
      ->generateCode();
  }
}
