<?php
// $Id: Scope.php,v 1.1.2.1 2011/01/04 18:25:40 pounard Exp $

/**
 * Scope context instance.
 */
class Xoxo_Code_Scope
{
  /**
   * Regex meant to be used with preg_replace() that will match all invalid
   * characters in PHP variable names.
   * 
   * @var string
   */
  const REGEX_VARIABLE_NAME = '/[^a-zA-Z0-9_]+/';

  /**
   * Current variables being used.
   * 
   * @var array
   */
  protected $_variables = array();

  /**
   * Helper for variable names generation.
   * 
   * @var int
   */
  protected $_variableCount = 1;

  /**
   * Escape a variable name, and change it in case the variable already being
   * used in current code.
   */
  public function createVariable($value = NULL) {
    // If no name given, create a new one.
    if (empty($value)) {
      $value = 'object';
    }

    // Escape it.
    $value = $this->escapeVariable($value);

    // If already exists, append the current counter value. We here use only
    // valid variable name characters, so we do not need to escape it twice.
    if (isset($this->_variables[$value])) {
      $value = $value .= (string) $this->_variableCount++;
    }

    // All done!
    return $value;
  }

  /**
   * Escape current variable name. If no value is given, we will create a new
   * value instead.
   */
  public function escapeVariable($value) {
    // If no name given, create a new one.
    if (empty($value)) {
      return $this->createVariable();
    }

    // Else simply escape current name. We won't check for collision here.
    $escaped = '$' . preg_replace(self::REGEX_VARIABLE_NAME, '', (string) $value);

    // Populate the variables array.
    $this->_variables[$escaped] = TRUE;

    return $escaped;
  }

  /**
   * Current indentation level.
   * 
   * @var int
   */
  protected $_indent = 0;

  /**
   * Get current indentation level.
   * 
   * @return int.
   */
  public function getIndent() {
    return $this->_indent;
  }

  /**
   * Increment current indentation level.
   * 
   * @return Xoxo_Code_Scope
   *   Self reference for chaining.
   */
  public function indent() {
    $this->_indent++;
    return $this;
  }

  /**
   * Decrement current indentation level.
   * 
   * @return Xoxo_Code_Scope
   *   Self reference for chaining.
   */
  public function unindent() {
    if ($this->_indent > 0) {
      $this->_indent--;
    }
    return $this;
  }
}
