<?php
// $Id: Exportable.php,v 1.1.2.1 2011/01/16 14:05:22 pounard Exp $

class Xoxo_Object_Exportable extends Xoxo_Object
{
  /**
   * @var string
   */
  protected $_name;

  /**
   * Set machine name.
   * 
   * @param string $name
   *   Machine name.
   * 
   * @return Xoxo_Object_Exportable
   *   Self reference for chaining.
   */
  public function setName($name) {
    $this->_name = $name;
    return $this;
  }

  /**
   * Get machine name.
   * 
   * @return string
   * 
   * @throws Oox_Exception
   *   If name not set.
   */
  public function getName() {
    if (!$this->_name) {
      throw new Oox_Exception("Name not set.");
    }
    return $this->_name;
  }

  /**
   * Default constructor.
   * 
   * @param string $name = NULL
   *   (optional) Machine name. Use this parameter if you intend to save or
   *   export your layout later.
   * @param string $description = NULL
   *   (optional) Human readable description.
   */
  public function __construct($name = NULL, $description = NULL) {
    parent::__construct($description);
    if ($name) {
      $this->_name = $name;
    }
  }

  /**
   * @see Xoxo_Object::duplicate()
   */
  public function duplicate() {
    $object = parent::duplicate();
    $object->setName(NULL);
    return $object;
  }

  /**
   * @see Oox_Formable_Interface::form()
   */
  public function formBuild(array &$form, array $values = array()) {
    parent::formBuild($form, $values);
    // Reorder elements.
    $form['xoxo']['description']['#weight'] = -1;
    // FIXME: We have to make this work.
    $form['xoxo']['name'] = array(
      '#type' => 'textfield',
//      '#type' => 'machine_name',
      '#title' => t("Name"),
      '#description' => t("This is the internal name. Only only alpha-numerical characters, and '_' or '-' are allowed."),
//      '#machine_name' => array(
//        'source' => array('description'),
//      ),
      '#weight' => -2,
    );
    // Disable this field for already saved objects. Name is another primary
    // key for exportables..
    if ($this->_oid) {
      $form['xoxo']['name']['#disabled'] = TRUE;
      $form['xoxo']['name']['#value'] = $this->_name;
    }  
  }

  /**
   * @see Oox_Formable_Interface::formValidate()
   */
  public function formValidate(array &$values) {
    if (!$errors = parent::formSubmit($values)) {
      $errors = array();
    }
    // Check the name matches the right formatting rules, only in case our
    // object is a new instance being created. Once the object is created no
    // name override will ever be possible.
    if (!$this->_oid) {
      if (!preg_match('/^[a-zA-Z0-9_-]+$/', trim($values['xoxo']['name']))) {
        $errors['xoxo']['name'] = t("Title contains illegal characters.");
      }
    }
    return $errors;
  }

  /**
   * @see Oox_Formable_Interface::formSubmit()
   */
  public function formSubmit(array &$values) {
    parent::formSubmit($values);
    if (!$this->_oid) {
      $this->_name = $values['xoxo']['name'];
    }
  }
}
